//------------------------------------------------------------
//                     ESP8266 ESP-12E
//                    __________________
//                   |    _   _   ___   |
//                   |   | |_| |_|      |
//                   |   |              |
//        TL. RESET -| RST          TXD |- 
//              BAT -| ADC          RXD |- 
//                  -| EN         GPIO5 |- 
//                  -| GPIO16     GPIO4 |- DHT22
//                  -| GPIO14     GPIO0 |- TL. PROG
//                  -| GPIO12     GPIO2 |- 
//              LED -| GPIO13    GPIO15 |- 
//            +3,3V -| Vcc          GND |- 0V
//                   |__________________|
//                     |  |  |  |  |  |
//------------------------------------------------------------

// knihovny
#include <ESP8266WiFi.h>
#include <DHT.h>
#include <ESP8266mDNS.h>
#include <WiFiUdp.h>
#include <ArduinoOTA.h>

// parametry
const char* ssid          = "****";
const char* pass          = "****";
const char* host          = "****";
const char* OTAhostname   = "id_205";
const char* id            = "205";

// piny procesoru
#define LED 13
#define AKU A0
#define pinDHT 4
#define PROG 0

// proměnné
float teplota = 0;
float vlhkost = 0;
float napeti;
int PocetPripojeni = 2;           // počet pokusů o odeslání dat na server před uspáním
int Pripojeni = PocetPripojeni;   // aktuální počet realizovaných pokusů o odeslání dat na server
unsigned long preMillis = 0;

#define typDHT22 DHT22
DHT mojeDHT(pinDHT, typDHT22);

//============================================================

void setup()
{
  // nastavení pinů procesoru
  pinMode(LED, OUTPUT);
  digitalWrite(LED, 1);
  pinMode(AKU, INPUT);
  pinMode(PROG, INPUT);

  // zahájení sériové komunikace
  Serial.begin(9600);
  Serial.println("");

//------------------------------------------------------------

    // start WiFi po probuzení
    WiFi.forceSleepBegin();
    delay(2000);

    // měření
    mojeDHT.begin();
    teplota = zmerTeplotu();
    vlhkost = zmerVlhkost();
    napeti = analogRead(AKU);

    // přepočet napětí
    napeti = napeti/170;

    // výpis na port
    Serial.print(napeti);
    Serial.println("V");
    Serial.print(teplota);
    Serial.println("st.");
    Serial.print(vlhkost);
    Serial.println("%");

    // korektní nastavení WiFi
    WiFi.forceSleepWake();
    delay(1);

//------------------------------------------------------------

  // připojení k WiFi

  // nastavíme ESP do režimu Station
  WiFi.mode(WIFI_STA);
  
  Serial.print("pripojovani k WiFi siti ");
  Serial.println(ssid);
  WiFi.begin(ssid,pass);
  
  // čekání na připojení k WiFi
  preMillis = millis();
  while (WiFi.status() != WL_CONNECTED)
  {
    if (millis() - preMillis >= 20000) {ESP.deepSleep(300e6);}  // po 20s bez připojení se ESP uspí
    Serial.print(".");
    digitalWrite(LED, 0); // LED blika
    delay(20);
    digitalWrite(LED, 1);
    delay(480);
  }
  Serial.println("");

//------------------------------------------------------------

  // po připojení k WiFi
  
  Serial.println("pripojeno k WiFi");

  // LED třikrát blikne
  digitalWrite(LED, 0);
  delay(20);
  digitalWrite(LED, 1);
  delay(150);
  digitalWrite(LED, 0);
  delay(20);
  digitalWrite(LED, 1);
  delay(150);
  digitalWrite(LED, 0);
  delay(20);
  digitalWrite(LED, 1);

  // pokud není stisknuté tlačítko PROG
  if (digitalRead(PROG) == 1)
  { 
    odeslaniDat();
    ESP.deepSleep(300e6);
  }

  // pokud je stisknuté tlačítko PROG
  if (digitalRead(PROG) == 0)
  {
    // spustíme OTA
    ArduinoOTA.setHostname(OTAhostname);
    
    ArduinoOTA.onStart([]()
    {
      Serial.println("Start");
    });
    ArduinoOTA.onEnd([]()
    {
      Serial.println("\nEnd");
    });
    ArduinoOTA.onProgress([](unsigned int progress, unsigned int total)
    {
      Serial.printf("Progress: %u%%\r", (progress / (total / 100)));
    });
    ArduinoOTA.onError([](ota_error_t error)
    {
      Serial.printf("Error[%u]: ", error);
      if (error == OTA_AUTH_ERROR) Serial.println("Auth Failed");
      else if (error == OTA_BEGIN_ERROR) Serial.println("Begin Failed");
      else if (error == OTA_CONNECT_ERROR) Serial.println("Connect Failed");
      else if (error == OTA_RECEIVE_ERROR) Serial.println("Receive Failed");
      else if (error == OTA_END_ERROR) Serial.println("End Failed");
    });
    ArduinoOTA.begin();
    digitalWrite(LED, 0);
  }
}
//============================================================

void loop()
{
  ArduinoOTA.handle();  // cyklická obsluha OTA
}

//============================================================

float zmerTeplotu()
{
  float teplota = mojeDHT.readTemperature();
  return teplota;
}

//------------------------------------------------------------

float zmerVlhkost()
{
  float vlhkost = mojeDHT.readHumidity();
  return vlhkost;
}

//------------------------------------------------------------

void odeslaniDat()
{
  // při výpadku WiFi přeruš
  if (WiFi.status() != WL_CONNECTED) {return;}

  // připojení k serveru
  const int httpPort = 80;
  Pripojeni = PocetPripojeni;

  pripoj:
  WiFiClient client;
  if (client.connect(host, httpPort))
  {
    long rssi = WiFi.RSSI();
    long ip = WiFi.localIP();
    
    // odeslání dat, pokud se spojení se serverem zdařilo
    Serial.println("odesilani");
    String url = "***/zapis_dat.php?id=";
    url += id;
    url += "&t=";
    url += teplota;
    url += "&v=";
    url += vlhkost;
    url += "&n=";
    url += napeti;
    url += "&s=";
    url += rssi;
    url += "&ip=";
    url += ip;
    
    client.print(String("GET ") + url + " HTTP/1.1\r\n" +
      "Host: " + host + "\r\n" +
      "Accept: text/html;q=0.9,image/webp,*/*;q=0.8" +
      "Connection: close\r\n\r\n");

    // čtení ze serveru
    delay(1000);
    while (client.available())
    {
      char c = client.read();
      Serial.print(c);
    }

    // odpojení klienta
    client.stop();
    Serial.println("odeslano");

    // 2x bliknutí LED
    digitalWrite(LED, 0);
    delay(20);
    digitalWrite(LED, 1);
    delay(150);
    digitalWrite(LED, 0);
    delay(20);
    digitalWrite(LED, 1);
  }
  else
  {
    // další pokusy, případně zrušení odesílání při nezdařeném spojení se serverem
    if (Pripojeni != 0)
    {
      Serial.println("pripojeni se nezdarilo, dalsi pokus");
      Pripojeni--;
      goto pripoj;
    }
  Serial.println("Vsechny pokusy o pripojeni selhaly.");
  Serial.println("Zarizeni bude uspano.");
  }
}
